@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- *  arch-tag: Implemetation files for podcast download manager
  *
  *  Copyright (C) 2005 Renato Araujo Oliveira Filho - INdT <renato.filho@indt.org.br>
  *
@@ -141,8 +139,7 @@ static guint rb_podcast_manager_signals[LAST_SIGNAL] = { 0 };
 /* functions */
 static void rb_podcast_manager_class_init 		(RBPodcastManagerClass *klass);
 static void rb_podcast_manager_init 			(RBPodcastManager *dp);
-static GObject *rb_podcast_manager_constructor 		(GType type, guint n_construct_properties,
-			   				 GObjectConstructParam *construct_properties);
+static void rb_podcast_manager_constructed		(GObject *object);
 static void rb_podcast_manager_dispose 			(GObject *object);
 static void rb_podcast_manager_finalize 		(GObject *object);
 static void rb_podcast_manager_set_property 		(GObject *object,
@@ -195,7 +192,7 @@ rb_podcast_manager_class_init (RBPodcastManagerClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	object_class->constructor = rb_podcast_manager_constructor;
+	object_class->constructed = rb_podcast_manager_constructed;
 	object_class->dispose = rb_podcast_manager_dispose;
 	object_class->finalize = rb_podcast_manager_finalize;
 
@@ -291,21 +288,14 @@ rb_podcast_manager_init (RBPodcastManager *pd)
 	eel_gconf_monitor_add (CONF_STATE_PODCAST_PREFIX);
 }
 
-static GObject *
-rb_podcast_manager_constructor (GType type, guint n_construct_properties,
-				GObjectConstructParam *construct_properties)
+static void
+rb_podcast_manager_constructed (GObject *object)
 {
-	RBPodcastManager *pd;
-
-	pd = RB_PODCAST_MANAGER (G_OBJECT_CLASS (rb_podcast_manager_parent_class)
-			->constructor (type, n_construct_properties, construct_properties));
-
+	RBPodcastManager *pd = RB_PODCAST_MANAGER (object);
+	RB_CHAIN_GOBJECT_METHOD (rb_podcast_manager_parent_class, constructed, object);
 	pd->priv->update_interval_notify_id = eel_gconf_notification_add (CONF_STATE_PODCAST_DOWNLOAD_INTERVAL,
 	                    			       			  rb_podcast_manager_config_changed,
 	                            		       			  pd);
-
-	return G_OBJECT (pd);
-
 }
 
 static void
@@ -1181,7 +1171,7 @@ rb_podcast_manager_add_post (RhythmDB *db,
 
 	/* initialize the rating */
 	g_value_init (&val, G_TYPE_DOUBLE);
-	g_value_set_double (&val, 2.5);
+	g_value_set_double (&val, 0.0);
 	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_RATING, &val);
 	g_value_unset (&val);
 
@@ -1807,6 +1797,7 @@ rb_podcast_manager_insert_feed (RBPodcastManager *pd, RBPodcastChannel *data)
 	GValue error_val = { 0, };
 	gulong last_post = 0;
 	gulong new_last_post;
+	const char *title;
 	GList *download_entries = NULL;
 	gboolean new_feed, updated, download_last;
 	RhythmDB *db = pd->priv->db;
@@ -1862,8 +1853,10 @@ rb_podcast_manager_insert_feed (RBPodcastManager *pd, RBPodcastChannel *data)
 	g_value_init (&title_val, G_TYPE_STRING);
 	if (data->title == NULL || strlen ((gchar *)data->title) == 0) {
 		g_value_set_string (&title_val, (gchar *) data->url);
+		title = data->url;
 	} else {
 		g_value_set_string (&title_val, (gchar *) data->title);
+		title = data->title;
 	}
 	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_TITLE, &title_val);
 	g_value_unset (&title_val);
@@ -1946,7 +1939,7 @@ rb_podcast_manager_insert_feed (RBPodcastManager *pd, RBPodcastChannel *data)
 
 			post_entry =
 				rb_podcast_manager_add_post (db,
-							     (gchar *) data->title,
+							     title,
 							     (gchar *) item->title,
 							     (gchar *) data->url,
 							     (gchar *) (item->author ? item->author : data->author),
