@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- *  arch-tag: Implementation of RhythmDB query result GtkTreeModel
  *
  *  Copyright (C) 2003 Colin Walters <walters@gnome.org>
  *
@@ -9,7 +7,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -35,9 +33,9 @@
 #include <string.h>
 #include <stdio.h>
 #include <math.h>
-#include <gsequence.h>
+#include <glib.h>
 
-#include <gtk/gtktreednd.h>
+#include <gtk/gtk.h>
 
 #include "rhythmdb-query-model.h"
 #include "rb-debug.h"
@@ -67,8 +65,7 @@ G_DEFINE_TYPE_WITH_CODE(RhythmDBQueryModel, rhythmdb_query_model, G_TYPE_OBJECT,
 					      rhythmdb_query_model_drag_dest_init))
 
 static void rhythmdb_query_model_init (RhythmDBQueryModel *shell_player);
-static GObject *rhythmdb_query_model_constructor (GType type, guint n_construct_properties,
-						  GObjectConstructParam *construct_properties);
+static void rhythmdb_query_model_constructed (GObject *object);
 static void rhythmdb_query_model_dispose (GObject *object);
 static void rhythmdb_query_model_finalize (GObject *object);
 static void rhythmdb_query_model_set_property (GObject *object,
@@ -291,7 +288,7 @@ rhythmdb_query_model_class_init (RhythmDBQueryModelClass *klass)
 
 	object_class->dispose = rhythmdb_query_model_dispose;
 	object_class->finalize = rhythmdb_query_model_finalize;
-	object_class->constructor = rhythmdb_query_model_constructor;
+	object_class->constructed = rhythmdb_query_model_constructed;
 
 	g_object_class_install_property (object_class,
 					 PROP_RHYTHMDB,
@@ -483,14 +480,12 @@ rhythmdb_query_model_set_query_internal (RhythmDBQueryModel *model,
 	rhythmdb_query_preprocess (model->priv->db, model->priv->query);
 
 	/* if the query contains time-relative criteria, re-run it periodically.
-	 * currently it's just every half hour, but perhaps it could be smarter.
+	 * currently it's just every minute, but perhaps it could be smarter.
 	 */
 	if (rhythmdb_query_is_time_relative (model->priv->db, model->priv->query)) {
 		if (model->priv->query_reapply_timeout_id == 0) {
 			model->priv->query_reapply_timeout_id =
-				g_timeout_add (1 * 60 * 1000,
-					       (GSourceFunc) rhythmdb_query_model_reapply_query_cb,
-					       model);
+				g_timeout_add_seconds (60, (GSourceFunc) rhythmdb_query_model_reapply_query_cb, model);
 		}
 	} else if (model->priv->query_reapply_timeout_id) {
 		g_source_remove (model->priv->query_reapply_timeout_id);
@@ -620,15 +615,13 @@ rhythmdb_query_model_init (RhythmDBQueryModel *model)
 	model->priv->reorder_drag_and_drop = FALSE;
 }
 
-static GObject *
-rhythmdb_query_model_constructor (GType type,
-				  guint n_construct_properties,
-				  GObjectConstructParam *construct_properties)
+static void
+rhythmdb_query_model_constructed (GObject *object)
 {
 	RhythmDBQueryModel *model;
 
-	model = RHYTHMDB_QUERY_MODEL (G_OBJECT_CLASS (rhythmdb_query_model_parent_class)->
-			constructor (type, n_construct_properties, construct_properties));
+	RB_CHAIN_GOBJECT_METHOD (rhythmdb_query_model_parent_class, constructed, object);
+	model = RHYTHMDB_QUERY_MODEL (object);
 
 	g_signal_connect_object (G_OBJECT (model->priv->db),
 				 "entry_added",
@@ -642,8 +635,6 @@ rhythmdb_query_model_constructor (GType type,
 				 "entry_deleted",
 				 G_CALLBACK (rhythmdb_query_model_entry_deleted_cb),
 				 model, 0);
-
-	return G_OBJECT (model);
 }
 
 static void
@@ -1764,6 +1755,7 @@ rhythmdb_query_model_drag_data_get (RbTreeDragSource *dragsource,
 {
 	RhythmDBQueryModel *model = RHYTHMDB_QUERY_MODEL (dragsource);
 	RhythmDBEntry *entry;
+	GdkAtom selection_data_target;
 	GString *data;
 	guint target;
 	GList *tem;
@@ -1771,8 +1763,9 @@ rhythmdb_query_model_drag_data_get (RbTreeDragSource *dragsource,
 
 	rb_debug ("getting drag data");
 
+	selection_data_target = gtk_selection_data_get_target (selection_data);
 	if (!gtk_target_list_find (rhythmdb_query_model_drag_target_list,
-				  selection_data->target, &target)) {
+				   selection_data_target, &target)) {
 		return FALSE;
 	}
 
@@ -1812,7 +1805,7 @@ rhythmdb_query_model_drag_data_get (RbTreeDragSource *dragsource,
 	}
 
 	gtk_selection_data_set (selection_data,
-				selection_data->target,
+				selection_data_target,
 				8, (guchar *) data->str,
 				data->len);
 
@@ -1858,7 +1851,8 @@ rhythmdb_query_model_drag_data_received (RbTreeDragDest *drag_dest,
 	if (model->priv->sort_func != NULL)
 		return FALSE;
 
-	if (selection_data->format == 8 && selection_data->length >= 0) {
+	if ((gtk_selection_data_get_format (selection_data) == 8) &&
+	    (gtk_selection_data_get_length (selection_data) >= 0)) {
 		GtkTreeIter iter;
 		GSequenceIter *ptr;
 		char **strv;
@@ -1866,9 +1860,9 @@ rhythmdb_query_model_drag_data_received (RbTreeDragDest *drag_dest,
 		gboolean uri_list;
 		int i = 0;
 
-		uri_list = (selection_data->type == gdk_atom_intern ("text/uri-list", TRUE));
+		uri_list = (gtk_selection_data_get_data_type (selection_data) == gdk_atom_intern ("text/uri-list", TRUE));
 
-		strv = g_strsplit ((char *) selection_data->data, "\r\n", -1);
+		strv = g_strsplit ((char *) gtk_selection_data_get_data (selection_data), "\r\n", -1);
 
 		if (dest == NULL || !rhythmdb_query_model_get_iter (GTK_TREE_MODEL (model), &iter, dest))
 			ptr = g_sequence_get_end_iter (model->priv->entries);
@@ -2648,8 +2642,14 @@ rhythmdb_query_model_album_sort_func (RhythmDBEntry *a,
 	gint ret;
 
 	/* Sort by album name */
-	a_val = rhythmdb_entry_get_string (a, RHYTHMDB_PROP_ALBUM_SORT_KEY);
-	b_val = rhythmdb_entry_get_string (b, RHYTHMDB_PROP_ALBUM_SORT_KEY);
+	a_val = rhythmdb_entry_get_string (a, RHYTHMDB_PROP_ALBUM_SORTNAME_SORT_KEY);
+	if (a_val[0] == '\0') {
+		a_val = rhythmdb_entry_get_string (a, RHYTHMDB_PROP_ALBUM_SORT_KEY);
+	}
+	b_val = rhythmdb_entry_get_string (b, RHYTHMDB_PROP_ALBUM_SORTNAME_SORT_KEY);
+	if (b_val[0] == '\0') {
+		b_val = rhythmdb_entry_get_string (b, RHYTHMDB_PROP_ALBUM_SORT_KEY);
+	}
 
 	if (a_val == NULL) {
 		if (b_val == NULL)
@@ -2702,8 +2702,14 @@ rhythmdb_query_model_artist_sort_func (RhythmDBEntry *a,
 	const char *b_val;
 	gint ret;
 
-	a_val = rhythmdb_entry_get_string (a, RHYTHMDB_PROP_ARTIST_SORT_KEY);
-	b_val = rhythmdb_entry_get_string (b, RHYTHMDB_PROP_ARTIST_SORT_KEY);
+	a_val = rhythmdb_entry_get_string (a, RHYTHMDB_PROP_ARTIST_SORTNAME_SORT_KEY);
+	if (a_val[0] == '\0') {
+		a_val = rhythmdb_entry_get_string (a, RHYTHMDB_PROP_ARTIST_SORT_KEY);
+	}
+	b_val = rhythmdb_entry_get_string (b, RHYTHMDB_PROP_ARTIST_SORTNAME_SORT_KEY);
+	if (b_val[0] == '\0') {
+		b_val = rhythmdb_entry_get_string (b, RHYTHMDB_PROP_ARTIST_SORT_KEY);
+	}
 
 	if (a_val == NULL) {
 		if (b_val == NULL)
@@ -2793,6 +2799,32 @@ rhythmdb_query_model_ulong_sort_func (RhythmDBEntry *a,
 		return rhythmdb_query_model_location_sort_func (a, b, data);
 }
 
+gint
+rhythmdb_query_model_bitrate_sort_func (RhythmDBEntry *a,
+					RhythmDBEntry *b,
+					gpointer data)
+{
+	gulong a_val, b_val;
+	
+	if (rhythmdb_entry_is_lossless (a)) {
+		if (rhythmdb_entry_is_lossless (b))
+			return rhythmdb_query_model_location_sort_func (a, b, data);
+		else
+			return 1;
+	} else {
+		if (rhythmdb_entry_is_lossless (b))
+			return -1;
+	}
+
+	a_val = rhythmdb_entry_get_ulong (a, RHYTHMDB_PROP_BITRATE);
+	b_val = rhythmdb_entry_get_ulong (b, RHYTHMDB_PROP_BITRATE);
+
+	if (a_val != b_val)
+		return (a_val > b_val ? 1 : -1);
+	else
+		return rhythmdb_query_model_location_sort_func (a, b, data);
+}
+
 gint
 rhythmdb_query_model_date_sort_func (RhythmDBEntry *a,
 				     RhythmDBEntry *b,
