@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- *  arch-tag: Implementation of Rhythmbox playlist management object
  *
  *  Copyright (C) 2003,2004 Colin Walters <walters@gnome.org>
  *
@@ -63,7 +61,7 @@
 #include "rhythmdb.h"
 #include "rb-stock-icons.h"
 #include "eel-gconf-extensions.h"
-#include "rb-glade-helpers.h"
+#include "rb-builder-helpers.h"
 #include "rb-util.h"
 
 #define RB_PLAYLIST_MGR_VERSION (xmlChar *) "1.0"
@@ -418,7 +416,7 @@ rb_playlist_manager_set_uimanager (RBPlaylistManager *mgr,
 	mgr->priv->uimanager = uimanager;
 
 	if (mgr->priv->actiongroup == NULL) {
-		mgr->priv->actiongroup = gtk_action_group_new ("PlaylistActions");
+		mgr->priv->actiongroup = gtk_action_group_new ("PlaylistManagerActions");
 		gtk_action_group_set_translation_domain (mgr->priv->actiongroup,
 							 GETTEXT_PACKAGE);
 		gtk_action_group_add_actions (mgr->priv->actiongroup,
@@ -1020,14 +1018,19 @@ static char *
 create_name_from_selection_data (RBPlaylistManager *mgr,
 				 GtkSelectionData *data)
 {
-	char  *name = NULL;
-	GList *list;
+	GdkAtom       type;
+	char         *name = NULL;
+	const guchar *selection_data_data;
+	GList        *list;
 
-        if (data->type == gdk_atom_intern ("text/uri-list", TRUE) ||
-	    data->type == gdk_atom_intern ("application/x-rhythmbox-entry", TRUE)) {
+	type = gtk_selection_data_get_data_type (data);
+	selection_data_data = gtk_selection_data_get_data (data);
+
+        if (type == gdk_atom_intern ("text/uri-list", TRUE) ||
+	    type == gdk_atom_intern ("application/x-rhythmbox-entry", TRUE)) {
 		gboolean is_id;
-		list = rb_uri_list_parse ((const char *)data->data);
-		is_id = (data->type == gdk_atom_intern ("application/x-rhythmbox-entry", TRUE));
+		list = rb_uri_list_parse ((const char *) selection_data_data);
+		is_id = (type == gdk_atom_intern ("application/x-rhythmbox-entry", TRUE));
 
 		if (list != NULL) {
 			GList   *l;
@@ -1099,7 +1102,7 @@ create_name_from_selection_data (RBPlaylistManager *mgr,
 	} else {
 		char **names;
 
-		names = g_strsplit ((char *)data->data, "\r\n", 0);
+		names = g_strsplit ((char *) selection_data_data, "\r\n", 0);
 		name = g_strjoinv (", ", names);
 		g_strfreev (names);
 	}
@@ -1127,11 +1130,14 @@ rb_playlist_manager_new_playlist_from_selection_data (RBPlaylistManager *mgr,
 						      GtkSelectionData *data)
 {
 	RBSource *playlist;
+	GdkAtom   type;
 	gboolean  automatic = TRUE;
 	char     *suggested_name;
 
-	if (data->type == gdk_atom_intern ("text/uri-list", TRUE) ||
-	    data->type == gdk_atom_intern ("application/x-rhythmbox-entry", TRUE))
+	type = gtk_selection_data_get_data_type (data);
+
+	if (type == gdk_atom_intern ("text/uri-list", TRUE) ||
+	    type == gdk_atom_intern ("application/x-rhythmbox-entry", TRUE))
 		automatic = FALSE;
 	suggested_name = create_name_from_selection_data (mgr, data);
 
@@ -1557,16 +1563,16 @@ setup_format_menu (GtkWidget* menu, GtkWidget *dialog)
 static void
 save_playlist (RBPlaylistManager *mgr, RBSource *source)
 {
-	GladeXML *xml;
-	GtkWidget *dialog, *menu;
-	char *name, *tmp;
+	GtkBuilder *builder;
+	GtkWidget *dialog;
+	GtkWidget *menu;
+	char *name;
+	char *tmp;
 
-	xml = rb_glade_xml_new ("playlist-save.glade",
-				"playlist_save_dialog",
-				mgr);
-	dialog = glade_xml_get_widget (xml, "playlist_save_dialog");
+	builder = rb_builder_load ("playlist-save.ui", mgr);
+	dialog = GTK_WIDGET (gtk_builder_get_object (builder, "playlist_save_dialog"));
 
-	menu = glade_xml_get_widget (xml, "playlist_format_menu");
+	menu = GTK_WIDGET (gtk_builder_get_object (builder, "playlist_format_menu"));
 	setup_format_menu (menu, dialog);
 	g_object_set_data (G_OBJECT (dialog), "export-menu", menu);
 
@@ -1582,7 +1588,7 @@ save_playlist (RBPlaylistManager *mgr, RBSource *source)
 				 G_CALLBACK (save_playlist_response_cb),
 				 source, 0);
 
-	g_object_unref (xml);
+	g_object_unref (builder);
 }
 
 static void
