@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- *  arch-tag: Implementation of search entry/browse toggle container
  *
  *  Copyright (C) 2003 Jorn Baayen <jorn@nl.linux.org>
  *  Copyright (C) 2003,2004 Colin Walters <walters@redhat.com>
@@ -534,6 +532,11 @@ rb_source_header_set_source_internal (RBSourceHeader *header,
 	} else {
 		/* no selected source -> hide source header */
 		gtk_widget_hide (GTK_WIDGET (header));
+		header->priv->have_browser = FALSE;
+		header->priv->disclosed = FALSE;
+
+		g_free (header->priv->browser_key);
+		header->priv->browser_key = NULL;
 	}
 
 	rb_source_header_refresh_search_bar (header);
@@ -674,6 +677,10 @@ rb_source_state_sync (RBSourceHeader *header,
 	gboolean do_search = FALSE;
 	char *old_text = NULL;
 
+	if (header->priv->selected_source == NULL) {
+		return;
+	}
+
 	state = g_hash_table_lookup (header->priv->source_states,
 				     header->priv->selected_source);
 	if (state == NULL) {
