@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- *  arch-tag: Implementation of the abstract base class of all sources
  *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
  *  Copyright (C) 2003 Colin Walters <walters@gnome.org>
@@ -78,6 +76,7 @@ static void default_get_status (RBSource *source, char **text, char **progress_t
 static void default_move_to_trash (RBSource *source);
 static GList * default_get_ui_actions (RBSource *source);
 static GList * default_get_search_actions (RBSource *source);
+static char *default_get_delete_action (RBSource *source);
 
 static void rb_source_post_entry_deleted_cb (GtkTreeModel *model,
 					     RhythmDBEntry *entry,
@@ -191,6 +190,7 @@ rb_source_class_init (RBSourceClass *klass)
 	klass->impl_get_status = default_get_status;
 	klass->impl_get_ui_actions = default_get_ui_actions;
 	klass->impl_get_search_actions = default_get_search_actions;
+	klass->impl_get_delete_action = default_get_delete_action;
 	klass->impl_move_to_trash = default_move_to_trash;
 
 	/**
@@ -367,7 +367,6 @@ rb_source_class_init (RBSourceClass *klass)
 							    RB_TYPE_SOURCE_SEARCH_TYPE,
 							    RB_SOURCE_SEARCH_NONE,
 							    G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-
 	/**
 	 * RBSource::deleted:
 	 * @source: the #RBSource
@@ -1575,6 +1574,29 @@ rb_source_get_search_actions (RBSource *source)
 	return klass->impl_get_search_actions (source);
 }
 
+static char *
+default_get_delete_action (RBSource *source)
+{
+	return g_strdup ("EditRemove");
+}
+
+/**
+ * rb_source_get_delete_action:
+ * @source: a #RBSource
+ *
+ * Returns the name of the UI action to use for 'delete'.
+ * This allows the source to customise the visible action name
+ * and description to better describe what deletion actually does.
+ *
+ * Return value: allocated string holding UI action name
+ */
+char *
+rb_source_get_delete_action (RBSource *source)
+{
+	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
+	return klass->impl_get_delete_action (source);
+}
+
 static gboolean
 _update_status_idle (RBSource *source)
 {
@@ -1658,6 +1680,26 @@ rb_source_gather_selected_properties (RBSource *source,
 	return tem;
 }
 
+static GtkActionGroup *
+find_action_group (GtkUIManager *uimanager, const char *group_name)
+{
+	GList *actiongroups;
+	GList *i;
+	actiongroups = gtk_ui_manager_get_action_groups (uimanager);
+
+	/* Don't create the action group if it's already registered */
+	for (i = actiongroups; i != NULL; i = i->next) {
+		const char *name;
+
+		name = gtk_action_group_get_name (GTK_ACTION_GROUP (i->data));
+		if (name != NULL && strcmp (name, group_name) == 0) {
+			return GTK_ACTION_GROUP (i->data);
+		}
+	}
+
+	return NULL;
+}
+
 /**
  * _rb_source_register_action_group:
  * @source: a #RBSource
@@ -1678,42 +1720,123 @@ _rb_source_register_action_group (RBSource *source,
 				  gpointer user_data)
 {
 	GtkUIManager *uimanager;
-	GList *actiongroups;
-	GList *i;
 	GtkActionGroup *group;
 
 	g_return_val_if_fail (group_name != NULL, NULL);
 
 	g_object_get (source, "ui-manager", &uimanager, NULL);
-	actiongroups = gtk_ui_manager_get_action_groups (uimanager);
-
-	/* Don't create the action group if it's already registered */
-	for (i = actiongroups; i != NULL; i = i->next) {
-		const char *name;
-
-		name = gtk_action_group_get_name (GTK_ACTION_GROUP (i->data));
-		if (name != NULL && strcmp (name, group_name) == 0) {
-			group = GTK_ACTION_GROUP (i->data);
-			/* Add a reference */
-			g_object_ref (group);
-			goto out;
+	group = find_action_group (uimanager, group_name);
+	if (group == NULL) {
+		group = gtk_action_group_new (group_name);
+		gtk_action_group_set_translation_domain (group,
+							 GETTEXT_PACKAGE);
+		if (actions != NULL) {
+			gtk_action_group_add_actions (group,
+						      actions, num_actions,
+						      user_data);
 		}
+		gtk_ui_manager_insert_action_group (uimanager, group, 0);
+	} else {
+		g_object_ref (group);
 	}
-
-	group = gtk_action_group_new (group_name);
-	gtk_action_group_set_translation_domain (group,
-						 GETTEXT_PACKAGE);
-	gtk_action_group_add_actions (group,
-				      actions, num_actions,
-				      user_data);
-	gtk_ui_manager_insert_action_group (uimanager, group, 0);
-
- out:
 	g_object_unref (uimanager);
 
 	return group;
 }
 
+typedef void (*SourceActionCallback) (GtkAction *action, RBSource *source);
+
+typedef struct {
+	SourceActionCallback callback;
+	/*RBShell *shell;*/
+	gpointer shell;
+} SourceActionData;
+
+static void
+source_action_data_destroy (SourceActionData *data)
+{
+	if (data->shell != NULL) {
+		g_object_remove_weak_pointer (G_OBJECT (data->shell), &data->shell);
+	}
+	g_slice_free (SourceActionData, data);
+}
+
+static void
+source_action_cb (GtkAction *action, SourceActionData *data)
+{
+	RBSource *source;
+
+	if (data->shell == NULL) {
+		return;
+	}
+
+	/* get current source */
+	g_object_get (G_OBJECT (data->shell), "selected-source", &source, NULL);
+	if (source != NULL) {
+		data->callback (action, source);
+		g_object_unref (source);
+	}
+}
+
+/**
+ * _rb_action_group_add_source_actions:
+ * @group: a #GtkActionGroup
+ * @shell: the #RBShell
+ * @actions: array of GtkActionEntry structures for the action group
+ * @num_actions: number of actions in the @actions array
+ *
+ * Adds actions to an action group where the action callback is
+ * called with the current selected source.  This can safely be called
+ * multiple times on the same action group.
+ */
+void
+_rb_action_group_add_source_actions (GtkActionGroup *group,
+				     GObject *shell,
+				     GtkActionEntry *actions,
+				     int num_actions)
+{
+	int i;
+	for (i = 0; i < num_actions; i++) {
+		GtkAction *action;
+		const char *label;
+		const char *tooltip;
+		SourceActionData *source_action_data;
+
+		if (gtk_action_group_get_action (group, actions[i].name) != NULL) {
+			/* action was already added */
+			continue;
+		}
+
+		label = gtk_action_group_translate_string (group, actions[i].label);
+		tooltip = gtk_action_group_translate_string (group, actions[i].tooltip);
+
+		action = gtk_action_new (actions[i].name, label, tooltip, NULL);
+		if (actions[i].stock_id != NULL) {
+			g_object_set (action, "stock-id", actions[i].stock_id, NULL);
+			if (gtk_icon_theme_has_icon (gtk_icon_theme_get_default (),
+						     actions[i].stock_id)) {
+				g_object_set (action, "icon-name", actions[i].stock_id, NULL);
+			}
+		}
+
+		if (actions[i].callback) {
+			GClosure *closure;
+			source_action_data = g_slice_new0 (SourceActionData);
+			source_action_data->callback = (SourceActionCallback) actions[i].callback;
+			source_action_data->shell = shell;
+			g_object_add_weak_pointer (shell, &source_action_data->shell);
+
+			closure = g_cclosure_new (G_CALLBACK (source_action_cb),
+						  source_action_data,
+						  (GClosureNotify) source_action_data_destroy);
+			g_signal_connect_closure (action, "activate", closure, FALSE);
+		}
+
+		gtk_action_group_add_action_with_accel (group, action, actions[i].accelerator);
+		g_object_unref (action);
+	}
+}
+
 /**
  * _rb_source_check_entry_type:
  * @source: a #RBSource
@@ -1738,6 +1861,30 @@ _rb_source_check_entry_type (RBSource *source, RhythmDBEntry *entry)
 	return ret;
 }
 
+/**
+ * _rb_source_set_import_status:
+ * @source: an #RBSource
+ * @job: a #RhythmDBImportJob
+ * @progress_text: used to return progress text
+ * @progress: used to return progress fraction
+ *
+ * Used in implementations of the get_status method to provide source
+ * status information based on a #RhythmDBImportJob.
+ */
+void
+_rb_source_set_import_status (RBSource *source, RhythmDBImportJob *job, char **progress_text, float *progress)
+{
+	int total;
+	int imported;
+
+	total = rhythmdb_import_job_get_total (job);
+	imported = rhythmdb_import_job_get_imported (job);
+
+	g_free (*progress_text);
+	*progress_text = g_strdup_printf (_("Importing (%d/%d)"), imported, total);
+	*progress = ((float)imported / (float)total);
+}
+
 /* This should really be standard. */
 #define ENUM_ENTRY(NAME, DESC) { NAME, "" #NAME "", DESC }
 
