@@ -1,6 +1,4 @@
 /*
- *  arch-tag: Implementation of RhythmDB query creation properties
- *
  *  Copyright (C) 2003, 2004 Colin Walters <walters@gnome.org>
  *  Copyright (C) 2005 James Livingston <walters@gnome.org>
  *
@@ -9,7 +7,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -30,9 +28,9 @@
 
 #include "config.h"
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
+#include <glib.h>
 #include <gtk/gtk.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 
 #include "rhythmdb.h"
 #include "rb-query-creator-private.h"
@@ -72,23 +70,23 @@ static void relativeTimeCriteriaGetWidgetData (GtkWidget *widget, GValue *val);
  */
 const RBQueryCreatorPropertyOption property_options[] =
 {
-	{ N_("Title"), RHYTHMDB_PROP_TITLE, RHYTHMDB_PROP_TITLE_FOLDED, &string_property_type },
-	{ N_("Artist"), RHYTHMDB_PROP_ARTIST, RHYTHMDB_PROP_ARTIST_FOLDED, &string_property_type },
-	{ N_("Album"), RHYTHMDB_PROP_ALBUM, RHYTHMDB_PROP_ALBUM_FOLDED, &string_property_type },
-	{ N_("Genre"), RHYTHMDB_PROP_GENRE, RHYTHMDB_PROP_GENRE_FOLDED, &string_property_type },
-	{ N_("Year"), RHYTHMDB_PROP_DATE, RHYTHMDB_PROP_DATE, &year_property_type },
-	{ N_("Rating"), RHYTHMDB_PROP_RATING, RHYTHMDB_PROP_RATING, &rating_property_type },
-	{ N_("Path"), RHYTHMDB_PROP_LOCATION, RHYTHMDB_PROP_LOCATION, &escaped_string_property_type },
+	{ NC_("query-criteria", "Title"), RHYTHMDB_PROP_TITLE, RHYTHMDB_PROP_TITLE_FOLDED, &string_property_type },
+	{ NC_("query-criteria", "Artist"), RHYTHMDB_PROP_ARTIST, RHYTHMDB_PROP_ARTIST_FOLDED, &string_property_type },
+	{ NC_("query-criteria", "Album"), RHYTHMDB_PROP_ALBUM, RHYTHMDB_PROP_ALBUM_FOLDED, &string_property_type },
+	{ NC_("query-criteria", "Genre"), RHYTHMDB_PROP_GENRE, RHYTHMDB_PROP_GENRE_FOLDED, &string_property_type },
+	{ NC_("query-criteria", "Year"), RHYTHMDB_PROP_DATE, RHYTHMDB_PROP_DATE, &year_property_type },
+	{ NC_("query-criteria", "Rating"), RHYTHMDB_PROP_RATING, RHYTHMDB_PROP_RATING, &rating_property_type },
+	{ NC_("query-criteria", "Path"), RHYTHMDB_PROP_LOCATION, RHYTHMDB_PROP_LOCATION, &escaped_string_property_type },
 
-	{ N_("Play Count"), RHYTHMDB_PROP_PLAY_COUNT, RHYTHMDB_PROP_PLAY_COUNT, &integer_property_type },
-	{ N_("Track Number"), RHYTHMDB_PROP_TRACK_NUMBER, RHYTHMDB_PROP_TRACK_NUMBER, &integer_property_type },
-	{ N_("Disc Number"), RHYTHMDB_PROP_DISC_NUMBER, RHYTHMDB_PROP_DISC_NUMBER, &integer_property_type },
-	{ N_("Bitrate"), RHYTHMDB_PROP_BITRATE, RHYTHMDB_PROP_BITRATE, &integer_property_type },
+	{ NC_("query-criteria", "Play Count"), RHYTHMDB_PROP_PLAY_COUNT, RHYTHMDB_PROP_PLAY_COUNT, &integer_property_type },
+	{ NC_("query-criteria", "Track Number"), RHYTHMDB_PROP_TRACK_NUMBER, RHYTHMDB_PROP_TRACK_NUMBER, &integer_property_type },
+	{ NC_("query-criteria", "Disc Number"), RHYTHMDB_PROP_DISC_NUMBER, RHYTHMDB_PROP_DISC_NUMBER, &integer_property_type },
+	{ NC_("query-criteria", "Bitrate"), RHYTHMDB_PROP_BITRATE, RHYTHMDB_PROP_BITRATE, &integer_property_type },
 
-	{ N_("Duration"), RHYTHMDB_PROP_DURATION, RHYTHMDB_PROP_DURATION, &duration_property_type },
+	{ NC_("query-criteria", "Duration"), RHYTHMDB_PROP_DURATION, RHYTHMDB_PROP_DURATION, &duration_property_type },
 
-	{ N_("Time of Last Play"), RHYTHMDB_PROP_LAST_PLAYED, RHYTHMDB_PROP_LAST_PLAYED, &relative_time_property_type },
-	{ N_("Time Added to Library"), RHYTHMDB_PROP_FIRST_SEEN, RHYTHMDB_PROP_FIRST_SEEN, &relative_time_property_type },
+	{ NC_("query-criteria", "Time of Last Play"), RHYTHMDB_PROP_LAST_PLAYED, RHYTHMDB_PROP_LAST_PLAYED, &relative_time_property_type },
+	{ NC_("query-criteria", "Time Added to Library"), RHYTHMDB_PROP_FIRST_SEEN, RHYTHMDB_PROP_FIRST_SEEN, &relative_time_property_type },
 };
 
 const int num_property_options = G_N_ELEMENTS (property_options);
@@ -99,17 +97,17 @@ const int num_property_options = G_N_ELEMENTS (property_options);
  */
 const RBQueryCreatorSortOption sort_options[] =
 {
-	{ N_("Artist"), "Artist", N_("_In reverse alphabetical order") },
-	{ N_("Album"), "Album", N_("_In reverse alphabetical order") },
-	{ N_("Genre"), "Genre", N_("_In reverse alphabetical order") },
-	{ N_("Title"), "Title", N_("_In reverse alphabetical order") },
-	{ N_("Rating"), "Rating", N_("W_ith more highly rated tracks first") },
-	{ N_("Play Count"), "PlayCount", N_("W_ith more often played songs first") },
-	{ N_("Year"), "Year", N_("W_ith newer tracks first") },
-	{ N_("Duration"), "Time", N_("W_ith longer tracks first") },
-	{ N_("Track Number"), "Track", N_("_In decreasing order")},
-	{ N_("Last Played"), "LastPlayed", N_("W_ith more recently played tracks first") },
-	{ N_("Date Added"), "FirstSeen", N_("W_ith more recently added tracks first") },
+	{ NC_("query-sort", "Artist"), "Artist", N_("_In reverse alphabetical order") },
+	{ NC_("query-sort", "Album"), "Album", N_("_In reverse alphabetical order") },
+	{ NC_("query-sort", "Genre"), "Genre", N_("_In reverse alphabetical order") },
+	{ NC_("query-sort", "Title"), "Title", N_("_In reverse alphabetical order") },
+	{ NC_("query-sort", "Rating"), "Rating", N_("W_ith more highly rated tracks first") },
+	{ NC_("query-sort", "Play Count"), "PlayCount", N_("W_ith more often played songs first") },
+	{ NC_("query-sort", "Year"), "Year", N_("W_ith newer tracks first") },
+	{ NC_("query-sort", "Duration"), "Time", N_("W_ith longer tracks first") },
+	{ NC_("query-sort", "Track Number"), "Track", N_("_In decreasing order")},
+	{ NC_("query-sort", "Last Played"), "LastPlayed", N_("W_ith more recently played tracks first") },
+	{ NC_("query-sort", "Date Added"), "FirstSeen", N_("W_ith more recently added tracks first") },
 };
 
 const int num_sort_options = G_N_ELEMENTS (sort_options);
@@ -164,12 +162,12 @@ const RBQueryCreatorCriteriaOption numeric_criteria_options[] =
 
 const RBQueryCreatorCriteriaOption year_criteria_options[] =
 {
+	/* Translators: this matches songs within 1-Jan-YEAR to 31-Dec-YEAR */
 	{ N_("in"), 1, RHYTHMDB_QUERY_PROP_YEAR_EQUALS },
-	/* matches if within 1-JAN-YEAR to 31-DEC-YEAR */
+	/* Translators: this matches songs after 31-Dec-YEAR */
 	{ N_("after"), 1, RHYTHMDB_QUERY_PROP_YEAR_GREATER },
-	/* matches if >= 31-DEC-YEAR */
+	/* Translators: this matches songs before 1-Jan-YEAR */
 	{ N_("before"), 1, RHYTHMDB_QUERY_PROP_YEAR_LESS }
-	/* matches if < 1-DEC-YEAR */
 };
 
 const RBQueryCreatorPropertyType rating_property_type =
@@ -283,7 +281,7 @@ stringCriteriaGetWidgetData (GtkWidget *widget, GValue *val)
 static void
 escapedStringCriteriaSetWidgetData (GtkWidget *widget, GValue *val)
 {
-	char *text = gnome_vfs_unescape_string (g_value_get_string (val), NULL);
+	char *text = g_uri_unescape_string (g_value_get_string (val), NULL);
 	gtk_entry_set_text (GTK_ENTRY (widget), text);
 	g_free (text);
 }
@@ -291,7 +289,7 @@ escapedStringCriteriaSetWidgetData (GtkWidget *widget, GValue *val)
 static void
 escapedStringCriteriaGetWidgetData (GtkWidget *widget, GValue *val)
 {
-	char *text = gnome_vfs_escape_host_and_path_string (gtk_entry_get_text (GTK_ENTRY (widget)));
+	char *text = g_uri_escape_string (gtk_entry_get_text (GTK_ENTRY (widget)), G_URI_RESERVED_CHARS_ALLOWED_IN_PATH, TRUE);
 
 	g_value_init (val, G_TYPE_STRING);
 	g_value_set_string (val, text);
@@ -367,7 +365,14 @@ integerCriteriaGetWidgetData (GtkWidget *widget, GValue *val)
 static GtkWidget *
 yearCriteriaCreateWidget (gboolean *constrain)
 {
-	return gtk_spin_button_new_with_range (0.0, (double)G_MAXINT, 1.0);
+	GtkWidget *spin;
+	GDate date = {0, };
+	spin = gtk_spin_button_new_with_range (0.0, (double)G_MAXINT, 1.0);
+
+	/* set it to the current year */
+	g_date_set_time_t (&date, time (NULL));
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin), g_date_get_year (&date));
+	return spin;
 }
 
 static void
@@ -469,12 +474,12 @@ durationCriteriaGetWidgetData (GtkWidget *widget, GValue *val)
  */
 
 static void
-update_time_unit_limits (GtkOptionMenu *menu, GtkWidget *spin_button)
+update_time_unit_limits (GtkComboBox *menu, GtkWidget *spin_button)
 {
 	/* set the range on the spin button so it can't overflow when
 	 * converted to seconds when we're constructing the query
 	 */
-	gulong mult = time_unit_options [gtk_option_menu_get_history (menu)].timeMultiplier;
+	gulong mult = time_unit_options [gtk_combo_box_get_active (menu)].timeMultiplier;
 	gtk_spin_button_set_range (GTK_SPIN_BUTTON (spin_button), 1, G_MAXINT / mult);
 }
 
@@ -482,19 +487,16 @@ static GtkWidget*
 create_time_unit_option_menu (const RBQueryCreatorTimeUnitOption *options,
 			     int length)
 {
-	GtkWidget *menu = gtk_menu_new ();
-	GtkWidget *option_menu = gtk_option_menu_new ();
+	GtkWidget *combo;
 	int i;
 
+	combo = gtk_combo_box_new_text ();
 	for (i = 0; i < length; i++) {
-		GtkWidget *menu_item = gtk_menu_item_new_with_label (_(options[i].name));
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _(options[i].name));
 	}
+	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);
 
-	gtk_widget_show_all (menu);
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
-
-	return option_menu;
+	return combo;
 }
 
 static GtkWidget *
@@ -508,11 +510,11 @@ relativeTimeCriteriaCreateWidget (gboolean *constrain)
 	box = GTK_BOX (gtk_hbox_new (FALSE, 6));
 
 	timeSpin = gtk_spin_button_new_with_range (1.0, G_MAXINT, 1.0);
-	gtk_box_pack_start_defaults (box, timeSpin);
+	gtk_box_pack_start (box, timeSpin, TRUE, TRUE, 0);
 
 	timeOption = create_time_unit_option_menu (time_unit_options, G_N_ELEMENTS (time_unit_options));
-	gtk_option_menu_set_history (GTK_OPTION_MENU (timeOption), time_unit_options_default);
-	gtk_box_pack_start_defaults (box, timeOption);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (timeOption), time_unit_options_default);
+	gtk_box_pack_start (box, timeOption, TRUE, TRUE, 0);
 	
 	g_signal_connect_object (timeOption, "changed",
 				 G_CALLBACK (update_time_unit_limits),
@@ -528,7 +530,7 @@ relativeTimeCriteriaSetWidgetData (GtkWidget *widget, GValue *val)
 	GtkBox *box = GTK_BOX (widget);
 
 	GtkSpinButton *timeSpin = GTK_SPIN_BUTTON (get_box_widget_at_pos (box, 0));
-	GtkOptionMenu *unitMenu = GTK_OPTION_MENU (get_box_widget_at_pos (box, 1));
+	GtkComboBox *unitMenu = GTK_COMBO_BOX (get_box_widget_at_pos (box, 1));
 
 	gulong time = g_value_get_ulong (val);
 	gulong unit = 0;
@@ -544,7 +546,7 @@ relativeTimeCriteriaSetWidgetData (GtkWidget *widget, GValue *val)
 	time = time / time_unit_options[unit].timeMultiplier;
 	g_assert (time < G_MAXINT);
 	/* set the time value and unit*/
-	gtk_option_menu_set_history (unitMenu, unit);
+	gtk_combo_box_set_active (unitMenu, unit);
 	gtk_spin_button_set_value (timeSpin, time);
 }
 
@@ -552,9 +554,9 @@ static void
 relativeTimeCriteriaGetWidgetData (GtkWidget *widget, GValue *val)
 {
 	GtkSpinButton *timeSpin = GTK_SPIN_BUTTON (get_box_widget_at_pos (GTK_BOX (widget), 0));
-	GtkOptionMenu *unitMenu = GTK_OPTION_MENU (get_box_widget_at_pos (GTK_BOX (widget), 1));
+	GtkComboBox *unitMenu = GTK_COMBO_BOX (get_box_widget_at_pos (GTK_BOX (widget), 1));
 
-	gulong timeMultiplier = time_unit_options [gtk_option_menu_get_history (unitMenu)].timeMultiplier;
+	gulong timeMultiplier = time_unit_options [gtk_combo_box_get_active (unitMenu)].timeMultiplier;
 	gint value = gtk_spin_button_get_value_as_int (timeSpin) * timeMultiplier;
 	g_assert (value >= 0);
 
